//
//
//  class devmixer :- abstracts I/O from the mixer device
// 
//  written by Simon Aldrich (simon@simonaldrich.co.uk) 
//
//////////////////////////////////////////////////////////////////

#include "devmixer.h"

// default constructor
devmixer::devmixer(char *mixerdevice)
{
	m_devmixer = 0;
	
	// attempt to open the mixer device
	m_devmixer = open(mixerdevice, O_RDWR, 0);
	
	// detect open error
	if (m_devmixer == -1)
	{
		perror(mixerdevice);
		exit(1);
	}

	init_mixer();
}

devmixer::~devmixer()
{
	// close the audio device
	if (m_devmixer > 0)
	{
		close(m_devmixer);
	}
}

int devmixer::init_mixer()
{
	// query the mixer capabilities
	
	if (ioctl(m_devmixer, SOUND_MIXER_READ_DEVMASK, &m_outmask) == -1)
	{
		perror("Unable to query mixer capabilities");
		exit(1);
	}
	
	if (ioctl(m_devmixer, SOUND_MIXER_READ_RECMASK, &m_inmask) == -1)
	{
		perror("Unable to query mixer record capabilities");
		exit(1);
	}

	if (ioctl(m_devmixer, SOUND_MIXER_READ_STEREODEVS, &m_stereomask) == -1)
	{
		perror("Unable to query mixer stereo capabilities");
		exit(1);
	}

	return 1;	
}

int devmixer::enum_outchannels()
{
	const char	*labels[] = SOUND_DEVICE_NAMES;
	int counter;
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
	{
		if (m_outmask & (1 << counter))
			printf("%s,", labels[counter]);
	}
	
	printf("\n");
	
	return m_outmask;
}

int devmixer::enum_inchannels()
{
	const char	*labels[] = SOUND_DEVICE_NAMES;
	int counter;
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
	{
		if (m_inmask & (1 << counter))
			printf("%s,", labels[counter]);
	}
	
	printf("\n");
	
	return m_outmask;
}

int devmixer::get_rec_channel()
{
	const char	*labels[] = SOUND_DEVICE_NAMES;
	int 		mask, counter;
	
	if (ioctl(m_devmixer, SOUND_MIXER_READ_RECSRC, &mask) == -1)
	{
		perror("Unable to retrieve recording source");
		exit(1);
	}
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
	{
		if (mask & (1 << counter))
		{
			//printf("%s\n", labels[counter]);
			break;
		}
	}
	
	return counter;
}

int devmixer::set_rec_channel(int channel)
{
	if (m_inmask & (1 << channel))
	{

		if (ioctl(m_devmixer, SOUND_MIXER_WRITE_RECSRC, &channel) == -1)
		{
			perror("Unable to set recording source");
			exit(1);
		}
	
	}
	
	return channel;
}

int devmixer::set_rec_channel(char *channel)
{
	int channum, counter, chanmask;
	const char	*labels[] = SOUND_DEVICE_NAMES;
	
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
		if (strcasecmp(channel, labels[counter]) == 0)
		{
			channum = counter;	
			break;
		}
	
	//printf("channum: %d - %s\n", channum, labels[channum]);
		
	chanmask = 1 << channum;
			
	if (m_inmask & (1 << channum))
	{
		if (ioctl(m_devmixer, SOUND_MIXER_WRITE_RECSRC, &chanmask) == -1)
		{
			perror("Unable to set recording source");
			exit(1);
		}
	
	}
	
	return channum;
}

int devmixer::get_mixer_level(int channel)
{
	int volume;
	
	if (ioctl(m_devmixer, MIXER_READ(channel), &volume) == -1)
	{
		perror("Unable to query mixer settings.");
		exit(1);
	}
	
	return volume;	
}

int devmixer::save_mixer_settings()
{
	const char	*labels[] = SOUND_DEVICE_NAMES;
	char		left, right;
	int 		counter, level;
	int			recdevice;
	
	recdevice = get_rec_channel();
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
	{
		if (m_outmask & (1 << counter))
		{
			printf("%s,", labels[counter]);
			level = get_mixer_level(counter);
			
			left = (level & 0xFF);
			right = (level & 0xFF00) >> 8;
			
			if (m_stereomask & (1 << counter))
				printf("%d,%d,", left, right);
			else
				printf("%d,%d,", left, left);
				
			if (counter == recdevice)
				printf("+");
			else
				printf("-");
		
			printf("\n");				
		}		
	}	
}

int devmixer::save_mixer_settings(char *filename)
{
	const char	*labels[] = SOUND_DEVICE_NAMES;
	char		left, right;
	int 		counter, level;
	int			recdevice;
	FILE		*outfile;
	
	outfile = fopen(filename, "wb+");
	
	recdevice = get_rec_channel();
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
	{
		if (m_outmask & (1 << counter))
		{
			fprintf(outfile, "%s,", labels[counter]);
			level = get_mixer_level(counter);
			
			left = (level & 0xFF);
			right = (level & 0xFF00) >> 8;
			
			if (m_stereomask & (1 << counter))
				fprintf(outfile, "%d,%d,", left, right);
			else
				fprintf(outfile, "%d,%d,", left, left);
				
			if (counter == recdevice)
				fprintf(outfile, "+");
			else
				fprintf(outfile, "-");
		
			fprintf(outfile, "\n");				
		}		
	}	
	fclose(outfile);
}

int devmixer::getcvstring(char *source, char *dest, int start)
{
	char 	*temp;
	int 	counter, counter2;
	
	temp = (char*) calloc(strlen(source), sizeof(char));
	counter2 = 0;
	for (counter = start; counter < strlen(source); counter++)
	{
		if (source[counter] != ',')
		{
			temp[counter2] = source[counter];
			counter2++;
		}
		else
		{
			temp[counter2] = '\0';
			counter++;
			break;
		}
	}

	strcpy(dest, temp);
	free(temp);
	
	return counter;
}

int devmixer::load_mixer_settings(char *filename)
{
	int			left, right, cvstart;
	char		*devname, *txtline, *temp, recsrc;
	FILE		*outfile;
	
	// open the stored settings file
	outfile = fopen(filename, "rb");
	if (outfile == NULL)
		return -1;
	
	// allocate a buffer to store a line of text in
	txtline = (char*) calloc(80, sizeof(char));
	if (txtline == NULL)
		return -1;
	
	while (fgets(txtline, 80, outfile) != NULL)
	{		
		// allocate space for the devicename string
		devname = (char*) malloc(strlen(txtline));
		temp = (char*) malloc(strlen(txtline));
		
		cvstart = getcvstring(txtline, devname, 0);
		cvstart = getcvstring(txtline, temp, cvstart);
		left = atoi(temp);
		cvstart = getcvstring(txtline, temp, cvstart);
		right = atoi(temp);
		cvstart = getcvstring(txtline, temp, cvstart);
		recsrc = temp[0];		
		
		set_mixer_level(devname, left, right);
		if (recsrc == '+')
			set_rec_channel(devname);
		
		free(temp);
		free(devname);
	}
	// free up the text buffer
	free(txtline);
	// close the external file
	fclose(outfile);		
}

int devmixer::set_mixer_level(int channel, char left, char right)
{
	int level;
	
	if (left < 0)
		left = 0;
		
	if (left > 100)
		left = 100;
		
	if (right < 0)
		right = 0;
		
	if (right > 100)
		right = 100;

	level = left + (right << 8);

	if (ioctl(m_devmixer, MIXER_WRITE(channel), &level) == -1)
	{
		perror("Unable to set mixer settings.");
		exit(1);
	}
	
	return level;		
}

int devmixer::set_mixer_level(char *channel, char left, char right)
{
	const char	*labels[] = SOUND_DEVICE_NAMES;
	int 		level, counter;
	
	if (left < 0)
		left = 0;
		
	if (left > 100)
		left = 100;
		
	if (right < 0)
		right = 0;
		
	if (right > 100)
		right = 100;

	level = left + (right << 8);
	
	for (counter = 0; counter < SOUND_MIXER_NRDEVICES; counter++)
	{
		if (strcasecmp(channel, labels[counter]) == 0)
		{
			if (ioctl(m_devmixer, MIXER_WRITE(counter), &level) == -1)
			{
				perror("Unable to set mixer settings.");
				exit(1);
			}
			break;
		}
	}
	
	return level;		
}
